// Copyright 2017 Drone.IO Inc. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package github

import (
	"net/url"
	"strconv"
	"strings"

	"github.com/jenkins-x/go-scm/scm"
)

// NormLogin normalizes GitHub login strings
var NormLogin = strings.ToLower

func encodeListOptions(opts scm.ListOptions) string {
	return encodeListOptionsWith(opts, url.Values{})
}

func encodeListOptionsWith(opts scm.ListOptions, params url.Values) string {
	if opts.Page != 0 {
		params.Set("page", strconv.Itoa(opts.Page))
	}
	if opts.Size != 0 {
		params.Set("per_page", strconv.Itoa(opts.Size))
	}
	return params.Encode()
}

func encodeCommitListOptions(opts scm.CommitListOptions) string {
	params := url.Values{}
	if opts.Page != 0 {
		params.Set("page", strconv.Itoa(opts.Page))
	}
	if opts.Size != 0 {
		params.Set("per_page", strconv.Itoa(opts.Size))
	}
	if opts.Ref != "" {
		params.Set("ref", opts.Ref)
	}
	return params.Encode()
}

func encodeIssueListOptions(opts scm.IssueListOptions) string {
	params := url.Values{}
	if opts.Page != 0 {
		params.Set("page", strconv.Itoa(opts.Page))
	}
	if opts.Size != 0 {
		params.Set("per_page", strconv.Itoa(opts.Size))
	}
	if opts.Open && opts.Closed {
		params.Set("state", "all")
	} else if opts.Closed {
		params.Set("state", "closed")
	}
	return params.Encode()
}

func encodeIssueSearchOptions(opts scm.SearchOptions) string {
	params := url.Values{}
	if opts.Sort == "" {
		opts.Sort = "created"
	}
	params.Set("sort", opts.Sort)
	order := "desc"
	if opts.Ascending {
		order = "asc"
	}
	params.Set("order", order)
	return params.Encode()
}

func encodePullRequestListOptions(opts scm.PullRequestListOptions) string {
	params := url.Values{}
	if opts.Page != 0 {
		params.Set("page", strconv.Itoa(opts.Page))
	}
	if opts.Size != 0 {
		params.Set("per_page", strconv.Itoa(opts.Size))
	}
	if opts.Open && opts.Closed {
		params.Set("state", "all")
	} else if opts.Closed {
		params.Set("state", "closed")
	}
	return params.Encode()
}

func encodePullRequestMergeOptions(opts *scm.PullRequestMergeOptions) *pullRequestMergeRequest {
	mr := &pullRequestMergeRequest{}
	if opts.MergeMethod != "" {
		mr.MergeMethod = opts.MergeMethod
	}
	if opts.CommitTitle != "" {
		mr.CommitTitle = opts.CommitTitle
	}
	if opts.SHA != "" {
		mr.SHA = opts.SHA
	}
	return mr
}
